<?php  
namespace Concrete\Package\CcOptinFree;

// Core C5 classes
use Package;
use Config;
use Loader;
use Database;
use Events;
use Route;
use UserInfo;
use Page;
use User;
use View;
use PermissionKey;
use Group;
use Concrete\Core\Permission\Access\Entity\GroupEntity as GroupPermissionAccessEntity;
use PermissionAccess;
use SinglePage;
use CollectionAttributeKey;
use URL;
use Concrete\Core\Asset\AssetList;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Http\ResponseAssetGroup;
use Concrete\Core\Application\Service\Urls;

use Concrete\Package\CcOptinFree\Controllers\CcOptinSecurity;
use Concrete\Package\CcOptinFree\Controllers\CampaignController;
use Concrete\Package\CcOptinFree\Controllers\ContentController;
use Concrete\Package\CcOptinFree\Controllers\SubscribeController;

use Concrete\Package\CcOptinFree\Contexts\DisplayCampaigns;

///// Application entrypoint. /////


class Controller extends Package
{
	private static $ccPackages = array('cc_optin_pro', 'cc_optin_lite', 'cc_optin_free');

	protected $appVersionRequired = '5.7.1';

	protected $pkgHandle = 'cc_optin_free';
	protected $pkgVersion = '0.9.13';
	protected $pkgName = 'Opt-In Free';

	public function __construct() {
		// Test if defined here to avoid package clashes when installing.
		if(!defined('CC_OPTIN')) {
			define('CC_OPTIN', $this->pkgHandle);
		}
	}

	public function getPackageDescription() {
		return t("Create great Opt-In email campaigns. NOTE: When upgrading to Lite or Pro, don't uninstall the previous version, just install the new one.");
	}

	private function getVersionType() {
		// NOTE: Part of product name, should not be translated.
		return t("Free");
	}

	public function getPackageName() {
		// NOTE: Product name, should not be translated.
		return t("Opt-In") . ' ' . $this->getVersionType();
	}

	public function install() {
		$pkg = parent::install();
		
		$packages = self::$ccPackages;
		unset($packages[array_search($this->pkgHandle, $packages)]);

		// Uninstall other cc_optin packages
		// Move config values to new package, then uninstall.
		foreach ($packages as $package) {
			$oldPkg = Package::getByHandle($package);
			if($oldPkg) {
				$db = Database::getActiveConnection();
				$db->executeUpdate(
					'UPDATE Config SET configNamespace = ? WHERE configNamespace = ?', 
					array($pkg->getPackageHandle(), $oldPkg->getPackageHandle())
				);

				$oldPkg->uninstall();
			}
		}

		$this->addSinglePages($pkg);

	}

	public function upgrade() {
		parent::upgrade();
		$pkg = Package::getByHandle($this->pkgHandle);

		$this->addSinglePages($pkg);

	}

	public function uninstall()	{
		parent::uninstall();

		// If no cc_optin packages are left, remove DB.
		$found = false;

		foreach (self::$ccPackages as $package) {
			if(Package::getByHandle($package)) {
				$found = true;
				break;
			}
		}

		if(!$found) {
			$db = Database::getActiveConnection();
			$dbXml = file_get_contents(dirname(__FILE__) . '/db.xml');

			preg_match_all('#<table.*?name="(\w+)"#', $dbXml, $matches);

			foreach($matches[1] as $table)
				$db->executeUpdate('DROP TABLE IF EXISTS ' . $table);

			// Temp 5.7 fix
			$query = $db->createQueryBuilder();
			$query->delete('Config');

			foreach(self::$ccPackages as $package)
				$query->orWhere("configNamespace = '$package'");

			$query->execute();
		}
	}

	///// Events /////

	public function on_start() {
		// Register routes for the ajax dialogs, must do that before on_page_view.
		Route::register('/cc_optin/campaign', function() {
			$controller = new CampaignController();
			$controller->execute();
		});

		Route::register('/cc_optin/content', function() {
			$controller = new ContentController();
			$controller->execute();
		});

		Route::register('/cc_optin/subscribe', function() {
			$controller = new SubscribeController();
			$controller->execute();
		});

		$assets = new RegisterAssets($this->getUser());
		$assets->register();

		$self = $this;
		Events::addListener('on_page_view', function($e) use ($self) {
			$ccOptin = new StartCcOptin($e->getPageObject(), $self->getUser(), $self);
			$ccOptin->start();
		});
	}

	public function getUser() {
		$user = new User();
		return $user;
	}

	///// Private methods /////

	private function addSinglePages($pkg) {
		// This one is just a placeholder for the dashboard
		$this->addSinglePage($pkg, '', $this->getPackageName());

		$this->addSinglePage($pkg, '/subscribers', 'Subscribers');
	}


	private function addSinglePage($pkg, $path, $name, $exclude_nav = false, $description = null) {
		$sp = null;
		$fullPath = '/dashboard/cc_optin' . $path;

		foreach (SinglePage::getListByPackage($pkg) as $page) {
			if($page->getCollectionPath() == $fullPath) {
				$sp = $page;
				break;
			}
		}

		if(!$sp) $sp = SinglePage::add($fullPath, $pkg);

		if ($sp != null) {
			$data = array();

			$data['cName'] = $name;
			if($description) $data['cDescription'] = $description;

			$sp->update($data);
			
			if($exclude_nav) $sp->setAttribute('exclude_nav', 1);
		}
	}
}

class StartCcOptin
{
	private $page;
	private $user;
	private $campaigns;

	public function __construct($page, $user, $controller) {
		$this->page = $page;
		$this->user = $user;
		$this->campaigns = new DisplayCampaigns($page, $controller);
	}

	public function start() {
		if($this->campaigns->count() == 0 && !$this->user_isOptInAdmin()) return;
		$this->page_requireCommonAssets();
	}

	///// User /////

	private function user_isOptInAdmin() {
		if(!$this->user->isLoggedIn()) return false;
		$security = new CcOptinSecurity();
		return $security->canRead();
	}

	///// Page /////

	private function page_addToolbarButton() {
		$ihm = Loader::helper('concrete/ui/menu');
		$uh = new Urls();

		$dialogSize = CcOptin::campaignDialogSize();

		$title = t('Edit Opt-In Campaign on this page');

		$ihm->addPageHeaderMenuItem('edit_campaign', CC_OPTIN, array(
			'icon'    => 'list-alt',
			'position' => 'left',
			'label' => $title,
			'href'            => URL::to('/cc_optin/campaign') . '?cID=' . $this->page->getCollectionId(),
			'linkAttributes'  => array(
				'dialog-title'    => t('Edit Campaign'),
				'id'              => 'cc-optin-edit-campaign',
				'dialog-modal'    => 'true',
				'dialog-width'    => $dialogSize[0],
				'dialog-height'   => $dialogSize[1],
				//'class'           => 'dialog-launch ccm-icon-edit ccm-menu-icon'
			)
		));

		$this->campaigns_display();
	}

	///// Campaigns /////

	private function campaigns_display() {
		if($this->page->isEditMode() || $this->page->isAdminArea()) return;
		$this->campaigns->display();
	}

	///// Page /////

	private function page_require($assetType, $assetHandle) {
		$assets = ResponseAssetGroup::get();
		$assets->requireAsset($assetType, $assetHandle);
	}

	private function page_requireCommonAssets() {
		$this->page_require('javascript', 'jquery');
		$this->page_require('javascript', 'jquery-cookie');
		$this->page_require('javascript-inline', 'cc-optin');

		if($this->user_isOptInAdmin())
			$this->page_requireEditAssets();
		else
			$this->campaigns_display();
	}

	private function page_requireEditAssets() {
		// Javascript translation must be first since the others may have translation strings.
		$this->page_require('javascript-inline', 'cc-js-i18n');
		$this->page_require('core/sitemap', false); // Required from 5.7.3 (concretePageSelector)
		$this->page_require('javascript', 'cc-ajaxform');
		$this->page_require('javascript', 'cc-actionform');
		$this->page_require('javascript-inline', 'cc-edit-campaign-spinner');

		$this->page_addToolbarButton();
	}
}

class RegisterAssets
{
	private $user;
	private $assets;

	public function __construct($user) {
		$this->user = $user;
		$this->assets = AssetList::getInstance();
	}

	public function register() {
		$this->assets_registerCommon();
	}

	///// User /////

	private function user_isOptInAdmin() {
		if(!$this->user->isLoggedIn()) return false;
		$security = new CcOptinSecurity();
		return $security->canRead();
	}

	///// Assets /////

	/**
	* Register an asset with the CC_OPTIN package.
	*/
	private function assets_registerLocal($assetType, $assetHandle, $filename, $args = array()) {
		$this->assets->register($assetType, $assetHandle, $filename, $args, CcOptin::package());
	}

	private function assets_registerCommon() {
		// A c5 resource that isn't registered
		$this->assets->register('javascript', 'jquery-cookie', 'js/jquery-cookie.js', array('combine' => false, 'minify' => false));

		// Create the namespace, which will be tested for existence when the header button loads.
		// (menu_items/edit_campaign/view.coffee)
		// The debounce function is used in window scroll events and on some dashboard pages.
		$js = <<<JS
window.ccOptin = {};
window.ccOptin.debounce = function(a,b,c){var d;return function(){var e=this,f=arguments;clearTimeout(d),d=setTimeout(function(){d=null,c||a.apply(e,f)},b),c&&!d&&a.apply(e,f)}};
JS;
		$this->assets_registerLocal('javascript-inline', 'cc-optin', $js, array('position' => Asset::ASSET_POSITION_HEADER));

		$this->assets_registerLocal('javascript', 'cc-actionform',       'js/actionform.js');
		$this->assets_registerLocal('javascript', 'cc-ajaxform',         'js/ajaxform.js');
		$this->assets_registerLocal('javascript', 'cc-scrollspy',        'js/cc-bootstrap-scrollspy.js');
		$this->assets_registerLocal('javascript', 'cc-display-campaign', 'js/display-campaign.js');
		$this->assets_registerLocal('javascript', 'cc-imgpreview',       'js/imgpreview-full-jquery.js');
		$this->assets_registerLocal('javascript', 'cc-placeholder',      'js/jquery-placeholder.js');
		$this->assets_registerLocal('javascript', 'cc-plainmodal',       'js/jquery-plainmodal.js');
		$this->assets_registerLocal('javascript', 'cc-timeago',          'js/jquery-timeago.js');

		$this->assets_registerLocal('css', 'cc-template-reset',          'templates/reset.css');

		if($this->user_isOptInAdmin())
			$this->assets_registerEdit();
	}

	private function assets_registerEdit() {
		$js = <<<JS
jQuery(function() { 
	jQuery('#cc-optin-edit-campaign')
	.prepend('<span class="spinner"><div class="double-bounce1"></div><div class="double-bounce2"></div></span>');
});
JS;
		$this->assets_registerLocal('javascript-inline', 'cc-edit-campaign-spinner', $js);

		ob_start();
		View::element('../js_i18n', null, CC_OPTIN);
		$this->assets_registerLocal('javascript-inline', 'cc-js-i18n', ob_get_clean(), array('position' => Asset::ASSET_POSITION_HEADER));
	}
}
